@@ -1,5 +1,6 @@
 /*********************************************************
- * Copyright (c) 2011,2014-2017,2019-2022 VMware, Inc. All rights reserved.
+ * Copyright (c) 2011-2024 Broadcom. All Rights Reserved.
+ * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -30,6 +31,7 @@
  *
  *********************************************************/
 
+
 #ifndef __ASYNC_SOCKET_VTABLE_H__
 #define __ASYNC_SOCKET_VTABLE_H__
 
@@ -41,12 +43,9 @@
 
 /*
  * If we change the AsyncSocketVTable, we also need to change the follow files:
- * lib/blastSockets/asyncProxySocket.c
  * lib/asyncsocket/asyncsocket.c
  * lib/asyncsocket/asyncWebSocket.c
  * lib/asyncsocket/asyncNamedPipe.c
- * lib/udpfec/fecAsyncSocket.c
- * lib/udpfec/fecAsyncSslSocket.c
  * devices/vsock/asyncVmciSocket.c
  */
 typedef struct AsyncSocketVTable {
@@ -108,6 +107,8 @@ typedef struct AsyncSocketVTable {
    int (*isSendBufferFull)(AsyncSocket *asock);
    int (*getNetworkStats)(AsyncSocket *asock,
                           AsyncSocketNetworkStats *stats);
+   int (*getSNIHostname)(AsyncSocket *asock,
+                         const char **sniHostname);
    int (*close)(AsyncSocket *asock);
    int (*closeWrite)(AsyncSocket *asock);
    int (*cancelRecv)(AsyncSocket *asock, int *partialRecvd, void **recvBuf,
