@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2019 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2022 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -42,6 +42,7 @@
    #include <winsock2.h>
    #include <ws2tcpip.h>
    #include "err.h"
+   #include "preference.h"
 #endif
 
 /*
@@ -113,6 +114,30 @@ Poll_InitWithImpl(const PollImpl *impl)
    pollImpl->Init();
 }
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Poll_IsInitialized --
+ *
+ *      Allows libraries to determine if they can use poll or not.
+ *
+ * Results:
+ *      TRUE if and only if poll is initialized.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+Poll_IsInitialized(void)
+{
+   return pollImpl != NULL;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -131,9 +156,10 @@ Poll_InitWithImpl(const PollImpl *impl)
 void
 Poll_Exit(void)
 {
-   pollImpl->Exit();
-
-   pollImpl = NULL;
+   if (pollImpl) {
+      pollImpl->Exit();
+      pollImpl = NULL;
+   }
 }
 
 
@@ -416,11 +442,17 @@ Poll_CB_RTimeRemove(PollerFunction f,
  *
  * PollSocketPairPrepare --
  *
- *      Do miscellaneous preparetion for the socket pair before connecting
+ *      Do miscellaneous preparation for the socket pair before connecting
  *
  * Results:
- *      Socket bound to a local address, and another set properly.
- *      TRUE if all preparetion succeed, otherwise FALSE.
+ *      The given 'dst' server socket is bound to (and listening on, if a
+ *      stream socket was requested) the specified 'addr' address, with the
+ *      address updated to contain the actual assigned port number if a
+ *      wildcard port was specified.
+ *
+ *      The given client 'src' socket is set into the specified blocking mode.
+ *
+ *      Returns TRUE if all preparation succeeds, otherwise FALSE.
  *
  * Side effects:
  *      None.
@@ -430,32 +462,36 @@ Poll_CB_RTimeRemove(PollerFunction f,
 
 static Bool
 PollSocketPairPrepare(Bool blocking,           // IN: blocking socket?
-                      SOCKET *src,             // IN: client side socket
+                      SOCKET src,              // IN: client side socket
                       SOCKET dst,              // IN: server side socket
-                      struct sockaddr *addr,   // IN: the address connected to
-                      int addrlen,             // IN: length of struct sockaddr
+                      struct sockaddr *addr,   // IN/OUT: server listener address
+                      int addrlen,             // IN: size of listener address
                       int socketCommType)      // IN: SOCK_STREAM or SOCK_DGRAM?
 {
    if (bind(dst, addr, addrlen) == SOCKET_ERROR) {
-      Log("%s: Could not bind socket.\n", __FUNCTION__);
+      Log("%s: Could not bind dst socket %d, error %d.\n",
+          __FUNCTION__, dst, WSAGetLastError());
       return FALSE;
    }
 
    if (!blocking) {
       unsigned long a = 1;
-      if (ioctlsocket(*src, FIONBIO, &a) == SOCKET_ERROR) {
-         Log("%s: Could not make socket non-blocking.\n", __FUNCTION__);
+      if (ioctlsocket(src, FIONBIO, &a) == SOCKET_ERROR) {
+         Log("%s: Could not make src socket %d non-blocking, error %d.\n",
+             __FUNCTION__, src, WSAGetLastError());
          return FALSE;
       }
    }
 
    if (socketCommType == SOCK_STREAM && listen(dst, 1) == SOCKET_ERROR) {
-      Log("%s: Could not listen on a socket.\n", __FUNCTION__);
+      Log("%s: Could not listen on dst socket %d, error %d.\n",
+          __FUNCTION__, dst, WSAGetLastError());
       return FALSE;
    }
 
    if (getsockname(dst, addr, &addrlen) == SOCKET_ERROR) {
-      Log("%s: getsockname() failed.\n", __FUNCTION__);
+      Log("%s: getsockname() failed for dst socket %d, error %d.\n",
+         __FUNCTION__, dst, WSAGetLastError());
       return FALSE;
    }
 
@@ -468,7 +504,7 @@ PollSocketPairPrepare(Bool blocking,           // IN: blocking socket?
  *
  * PollSocketPairConnect --
  *
- *      Connects a socket to a given address.
+ *      Connects a given socket to a given address.
  *
  * Results:
  *      TRUE if connecting successfully, otherwise FALSE is returned.
@@ -481,18 +517,78 @@ PollSocketPairPrepare(Bool blocking,           // IN: blocking socket?
 
 static Bool
 PollSocketPairConnect(Bool blocking,           // IN: blocking socket?
-                      struct sockaddr *addr,   // IN: the address connected to
-                      int addrlen,             // IN: length of struct sockaddr
-                      SOCKET *s)               // IN: connecting socket
+                      struct sockaddr *addr,   // IN: the address to connect to
+                      int addrlen,             // IN: size of 'addr'
+                      SOCKET sock,             // IN: connecting socket
+                      SocketSpecialOpts opts)  // IN: socket special options
 {
-   if (connect(*s, addr, addrlen) == SOCKET_ERROR) {
-      if (blocking || WSAGetLastError() != WSAEWOULDBLOCK) {
-         Log("%s: Could not connect to a local socket.\n", __FUNCTION__);
+   if (blocking && (opts & POLL_OPTIONS_SOCKET_PAIR_NONBLOCK_CONN)) {
+      /* Change blocking socket to non-blocking socket for timeout */
+      unsigned long unblock = 1;
+      if (ioctlsocket(sock, FIONBIO, &unblock) == SOCKET_ERROR) {
+         Log("%s: Set socket %d to non-blocking mode failed, error %d.\n",
+             __FUNCTION__, sock, WSAGetLastError());
          return FALSE;
       }
-   } else if (!blocking) {
-      Log("%s: non-blocking socket connected immediately!\n", __FUNCTION__);
-      return FALSE;
+
+      if (connect(sock, addr, addrlen) == SOCKET_ERROR) {
+         WSAPOLLFD pollFds[1];
+         /* wait timeout seconds */
+         unsigned int timeout = Preference_GetLong(3,
+            "pref.wsa.socket.pair.connect.timeout.seconds") * 1000;
+         int ret = WSAGetLastError();
+         if (ret != WSAEWOULDBLOCK) {
+            /* connection failed */
+            Log("%s: Non-blocking socket %d could not connect to a local "
+                "socket, error %d.\n", __FUNCTION__, sock, WSAGetLastError());
+            return FALSE;
+         }
+
+         pollFds[0].fd = sock;
+         pollFds[0].events = POLLWRNORM;
+         pollFds[0].revents = 0;
+
+         ret = WSAPoll(pollFds, 1, timeout);
+         if (ret <= 0) {
+            /* WSAPoll failed or connection timed out */
+            if (ret == 0) {
+                WSASetLastError(WSAETIMEDOUT);
+            }
+            Log("%s: Non-blocking socket %d connect to listener socket "
+                "failed, error %d.\n", __FUNCTION__, sock, WSAGetLastError());
+            return FALSE;
+         }
+
+         if ((pollFds[0].revents &
+            (POLLWRNORM|POLLERR|POLLHUP)) != POLLWRNORM) {
+            /* connection failed */
+            int error = 0;
+            int errLen = sizeof error;
+            getsockopt(sock, SOL_SOCKET, SO_ERROR, (char *)&error, &errLen);
+            WSASetLastError(error);
+            Log("%s: Non-blocking socket %d connect to listener socket "
+                "failed, error %d.\n", __FUNCTION__, sock, WSAGetLastError());
+            return FALSE;
+         }
+      }
+      /* connection successful */
+      Log("%s: Non-blocking socket %d connected successfully with "
+          "socket type %d.\n", __FUNCTION__, sock, addr->sa_family);
+      unblock = 0;
+      if (ioctlsocket(sock, FIONBIO, &unblock) == SOCKET_ERROR) {
+         Log("%s: Non-blocking socket %d restored to blocking mode failed, "
+             "error %d.\n", __FUNCTION__, sock, WSAGetLastError());
+         return FALSE;
+      }
+   } else if (connect(sock, addr, addrlen) == SOCKET_ERROR) {
+      if (blocking || WSAGetLastError() != WSAEWOULDBLOCK) {
+         Log("%s: socket %d could not connect to a local socket, "
+             "error %d.\n", __FUNCTION__, sock, WSAGetLastError());
+         return FALSE;
+      }
+   } else {
+      Log("%s: %s socket %d connected immediately!\n",
+          __FUNCTION__, (blocking ? "Blocking" : "Non-blocking"), sock);
    }
 
    return TRUE;
@@ -544,49 +640,54 @@ PollSocketClose(SOCKET sock) {  // IN: the socket is being closed
  */
 
 static SOCKET
-PollSocketPairConnecting(sa_family_t sa_family,    // IN: socket family type
-                         int socketCommType,       // IN: SOCK_STREAM or SOCK_DGRAM?
-                         struct sockaddr *addr,    // IN: the address connected to
-                         int addrlen,              // IN: length of struct sockaddr
-                         Bool blocking,            // IN: blocking socket?
-                         SOCKET *s)                // OUT: connecting socket
+PollSocketPairConnecting(sa_family_t sa_family,  // IN: socket family type
+                         int socketCommType,     // IN: SOCK_STREAM or SOCK_DGRAM?
+                         struct sockaddr *addr,  // IN: the address connected to
+                         int addrlen,            // IN: length of struct sockaddr
+                         Bool blocking,          // IN: blocking socket?
+                         SOCKET *s,              // OUT: connected client socket
+                         SocketSpecialOpts opts) // IN: socket special options
 {
-   SOCKET temp = INVALID_SOCKET;
+   SOCKET server = INVALID_SOCKET; // server (listening) socket
 
-   *s = socket(sa_family, socketCommType, 0);
-   if (*s == INVALID_SOCKET) {
-      Log("%s: Could not create socket, socket family: %d.\n", __FUNCTION__,
-          sa_family);
-      goto out;
-   }
-
-   temp = socket(sa_family, socketCommType, 0);
-   if (temp == INVALID_SOCKET) {
-      PollSocketClose(*s);
-      *s = INVALID_SOCKET;
-      Log("%s: Could not create second socket, socket family: %d.\n",
+   SOCKET client = socket(sa_family, socketCommType, 0);
+   if (client == INVALID_SOCKET) {
+      Log("%s: Could not create client socket, socket family: %d.\n",
           __FUNCTION__, sa_family);
-      goto out;
+      goto outError;
    }
 
-   if (!PollSocketPairPrepare(blocking, s, temp, addr, addrlen, socketCommType)) {
-      Log("%s: Could not prepare the socket pair for the following connecting,\
-          socket type: %d\n", __FUNCTION__, sa_family);
-      goto outCloseTemp;
-   }
-
-   if (!PollSocketPairConnect(blocking, addr, addrlen, s)) {
-      Log("%s: Could not make socket pair connected, socket type: %d",
+   server = socket(sa_family, socketCommType, 0);
+   if (server == INVALID_SOCKET) {
+      Log("%s: Could not create server socket, socket family: %d.\n",
           __FUNCTION__, sa_family);
-      goto outCloseTemp;
+      goto outError;
    }
 
-   return temp;
+   if (!PollSocketPairPrepare(blocking, client, server, addr, addrlen, socketCommType)) {
+      Log("%s: Could not prepare socket pair for type %d, client %d, "
+          "server %d.\n", __FUNCTION__, sa_family, client, server);
+      goto outError;
+   }
 
-outCloseTemp:
-   PollSocketClose(temp);
+   if (!PollSocketPairConnect(blocking, addr, addrlen, client, opts)) {
+      Log("%s: Could not connect socket pair for type %d, client %d, "
+          "server %d.\n", __FUNCTION__, sa_family, client, server);
+      goto outError;
+   }
 
-out:
+   *s = client;
+   return server;
+
+outError:
+   if (server != INVALID_SOCKET) {
+      PollSocketClose(server);
+   }
+   if (client != INVALID_SOCKET) {
+      PollSocketClose(client);
+   }
+
+   *s = INVALID_SOCKET;
    return INVALID_SOCKET;
 }
 
@@ -600,9 +701,12 @@ out:
  *      creates an *IPv4* socket pair.
  *
  * Results:
- *      Socket bound to a local address, and another connecting
- *      to that address.
- *      INVALID_SOCKET on error.  Use WSAGetLastError() for detail.
+ *      On success, returns a listening socket bound to a local address, and
+ *      sets a passed-by-reference argument 's' to a client socket that has
+ *      connected to the listening socket.  The client socket's blocking mode
+ *      is as specified by the 'blocking' argument.
+ *
+ *      Returns INVALID_SOCKET on error.  Use WSAGetLastError() for detail.
  *
  * Side effects:
  *      None.
@@ -611,9 +715,10 @@ out:
  */
 
 static SOCKET
-PollIPv4SocketPairStartConnecting(int socketCommType,  // IN: SOCK_STREAM or SOCK_DGRAM?
-                                  Bool blocking,       // IN: blocking socket?
-                                  SOCKET *s)           // OUT: connecting socket
+PollIPv4SocketPairStartConnecting(int socketCommType,     // IN: SOCK_STREAM or SOCK_DGRAM?
+                                  Bool blocking,          // IN: blocking socket?
+                                  SOCKET *s,              // OUT: connecting socket
+                                  SocketSpecialOpts opts) // IN: socket special options
 {
    struct sockaddr_in iaddr;
    int addrlen;
@@ -625,7 +730,8 @@ PollIPv4SocketPairStartConnecting(int socketCommType,  // IN: SOCK_STREAM or SOC
    iaddr.sin_port = 0;
 
    return PollSocketPairConnecting(iaddr.sin_family, socketCommType,
-                                   (struct sockaddr*) &iaddr, addrlen, blocking, s);
+                                   (struct sockaddr*) &iaddr, addrlen,
+                                   blocking, s, opts);
 }
 
 
@@ -638,9 +744,12 @@ PollIPv4SocketPairStartConnecting(int socketCommType,  // IN: SOCK_STREAM or SOC
  *      creates an *IPv6* socket pair.
  *
  * Results:
- *      Socket bound to a local address, and another connecting
- *      to that address.
- *      INVALID_SOCKET on error.  Use WSAGetLastError() for detail.
+ *      On success, returns a listening socket bound to a local address, and
+ *      sets a passed-by-reference argument 's' to a client socket that has
+ *      connected to the listening socket.  The client socket's blocking mode
+ *      is as specified by the 'blocking' argument.
+ *
+ *      Returns INVALID_SOCKET on error.  Use WSAGetLastError() for detail.
  *
  * Side effects:
  *      None.
@@ -649,9 +758,10 @@ PollIPv4SocketPairStartConnecting(int socketCommType,  // IN: SOCK_STREAM or SOC
  */
 
 static SOCKET
-PollIPv6SocketPairStartConnecting(int socketCommType,  // IN: SOCK_STREAM or SOCK_DGRAM?
-                                  Bool blocking,       // IN: blocking socket?
-                                  SOCKET *s)           // OUT: connecting socket
+PollIPv6SocketPairStartConnecting(int socketCommType,     // IN: SOCK_STREAM or SOCK_DGRAM?
+                                  Bool blocking,          // IN: blocking socket?
+                                  SOCKET *s,              // OUT: connecting socket
+                                  SocketSpecialOpts opts) // IN: socket special options
 {
    struct sockaddr_in6 iaddr6;
    int addrlen;
@@ -663,7 +773,8 @@ PollIPv6SocketPairStartConnecting(int socketCommType,  // IN: SOCK_STREAM or SOC
    iaddr6.sin6_port = 0;
 
    return PollSocketPairConnecting(iaddr6.sin6_family, socketCommType,
-                                   (struct sockaddr*) &iaddr6, addrlen, blocking, s);
+                                   (struct sockaddr*) &iaddr6, addrlen,
+                                   blocking, s, opts);
 }
 
 
@@ -676,9 +787,12 @@ PollIPv6SocketPairStartConnecting(int socketCommType,  // IN: SOCK_STREAM or SOC
  *      creates a *VMCI* socket pair.
  *
  * Results:
- *      Socket bound to a local address, and another connecting
- *      to that address.
- *      INVALID_SOCKET on error.  Use WSAGetLastError() for detail.
+ *      On success, returns a listening socket bound to a local address, and
+ *      sets a passed-by-reference argument 's' to a client socket that has
+ *      connected to the listening socket.  The client socket's blocking mode
+ *      is as specified by the 'blocking' argument.
+ *
+ *      Returns INVALID_SOCKET on error.  Use WSAGetLastError() for detail.
  *
  * Side effects:
  *      None.
@@ -702,7 +816,8 @@ PollVMCISocketPairStartConnecting(int socketCommType,  // IN: SOCK_STREAM or SOC
    vaddr.svm_cid = VMCISock_GetLocalCID();
 
    return PollSocketPairConnecting(vaddr.svm_family, socketCommType,
-                                   (struct sockaddr*) &vaddr, addrlen, blocking, s);
+                                   (struct sockaddr*) &vaddr, addrlen,
+                                   blocking, s, 0);
 }
 
 
@@ -726,10 +841,11 @@ PollVMCISocketPairStartConnecting(int socketCommType,  // IN: SOCK_STREAM or SOC
  */
 
 static SOCKET
-PollSocketPairStartConnecting(Bool vmci,      // IN: vmci socket?
-                              Bool stream,    // IN: stream socket?
-                              Bool blocking,  // IN: blocking socket?
-                              SOCKET *s)      // OUT: connecting socket
+PollSocketPairStartConnecting(Bool vmci,              // IN: vmci socket?
+                              Bool stream,            // IN: stream socket?
+                              Bool blocking,          // IN: blocking socket?
+                              SOCKET *s,              // OUT: connecting socket
+                              SocketSpecialOpts opts) // IN: socket special options
 {
    SOCKET temp = INVALID_SOCKET;
    int socketCommType = stream ? SOCK_STREAM : SOCK_DGRAM;
@@ -737,13 +853,14 @@ PollSocketPairStartConnecting(Bool vmci,      // IN: vmci socket?
    if (vmci) {
       temp = PollVMCISocketPairStartConnecting(socketCommType, blocking, s);
    } else {
-      temp = PollIPv6SocketPairStartConnecting(socketCommType, blocking, s);
+      temp = PollIPv6SocketPairStartConnecting(socketCommType, blocking,
+                                               s, opts);
 
       if (temp == INVALID_SOCKET) {
-         temp = PollIPv4SocketPairStartConnecting(socketCommType, blocking, s);
+         temp = PollIPv4SocketPairStartConnecting(socketCommType, blocking,
+                                                  s, opts);
       }
    }
-
    return temp;
 }
 
@@ -765,35 +882,38 @@ PollSocketPairStartConnecting(Bool vmci,      // IN: vmci socket?
  */
 
 int
-Poll_SocketPair(Bool vmci,     // IN: create vmci pair?
-                Bool stream,   // IN: stream socket?
-                int fds[2])    // OUT: 2 sockets connected to each other
+Poll_SocketPair(Bool vmci,              // IN: create vmci pair?
+                Bool stream,            // IN: stream socket?
+                int fds[2],             // OUT: 2 sockets connected to each other
+                SocketSpecialOpts opts) // IN: socket special options
 {
-   SOCKET temp = INVALID_SOCKET;
+   SOCKET server = INVALID_SOCKET;
 
    fds[0] = INVALID_SOCKET;
    fds[1] = INVALID_SOCKET;
 
-   temp = PollSocketPairStartConnecting(vmci, stream, TRUE, (SOCKET *)&fds[0]);
-   if (temp == INVALID_SOCKET) {
+   server = PollSocketPairStartConnecting(vmci, stream, TRUE,
+                                          (SOCKET *)&fds[0], opts);
+   if (server == INVALID_SOCKET) {
       goto out;
    }
    if (stream) {
-      fds[1] = accept(temp, NULL, NULL);
+      fds[1] = accept(server, NULL, NULL);
       if (fds[1] == INVALID_SOCKET) {
-         Log("%s: Could not accept on a local socket.\n", __FUNCTION__);
+         Log("%s: Could not accept on a server socket %d.\n",
+             __FUNCTION__, server);
          goto out;
       }
-      closesocket(temp);
+      closesocket(server);
    } else {
-      fds[1] = temp;
+      fds[1] = server;
    }
    return 0;
 
 out:
    Warning("%s: Error creating a %s socket pair: %d/%s\n", __FUNCTION__,
            vmci ? "vmci" : "inet", WSAGetLastError(), Err_ErrString());
-   closesocket(temp);
+   closesocket(server);
    closesocket(fds[0]);
    closesocket(fds[1]);
    return SOCKET_ERROR;
@@ -2051,12 +2171,12 @@ PollUnitTest_StateMachine(void *clientData) // IN: Unused
 #ifdef _WIN32
          socketPairs[i].fds[0] = INVALID_SOCKET;
          socketPairs[i].fds[1] = INVALID_SOCKET;
-         if (Poll_SocketPair(useVMCI, TRUE, socketPairs[i].fds) < 0) {
+         if (Poll_SocketPair(useVMCI, TRUE, socketPairs[i].fds, 0) < 0) {
             Warning("%s:   failure -- error creating socketpair, iteration %d\n",
                     __FUNCTION__, i);
             break;
          }
-         send(socketPairs[i].fds[0], (const char *)fds, sizeof fds, 0);
+         retval = send(socketPairs[i].fds[0], (const char *)fds, sizeof fds, 0);
 #else
          int addrFamily = useVMCI ? VMCISock_GetAFValue(): AF_UNIX;
          socketPairs[i].fds[0] = -1;
@@ -2128,13 +2248,13 @@ PollUnitTest_StateMachine(void *clientData) // IN: Unused
          closesocket(fds[1]);
          fds[0] = INVALID_SOCKET;
          fds[1] = INVALID_SOCKET;
-         if (Poll_SocketPair(TRUE, TRUE, fds) < 0) {
+         if (Poll_SocketPair(TRUE, TRUE, fds, 0) < 0) {
             Warning("%s:   failure -- error creating vmci socketpair\n",
                     __FUNCTION__);
             state ++;
             break;
          }
-         send(fds[0], (const char *)fds, sizeof fds, 0);
+         retval = send(fds[0], (const char *)fds, sizeof fds, 0);
    #else
          close(fds[0]);
          close(fds[1]);
@@ -2171,13 +2291,13 @@ PollUnitTest_StateMachine(void *clientData) // IN: Unused
          closesocket(fds[1]);
          fds[0] = INVALID_SOCKET;
          fds[1] = INVALID_SOCKET;
-         if (Poll_SocketPair(FALSE, TRUE, fds) < 0) {
+         if (Poll_SocketPair(FALSE, TRUE, fds, 0) < 0) {
             Warning("%s:   failure -- error creating socketpair\n",
                     __FUNCTION__);
             state += 3;
             break;
          }
-         send(fds[0], (const char *)fds, sizeof fds, 0);
+         retval = send(fds[0], (const char *)fds, sizeof fds, 0);
    #endif
          state = 0;
          break;
@@ -2201,7 +2321,7 @@ PollUnitTest_StateMachine(void *clientData) // IN: Unused
       closesocket(fds[1]);
       fds[0] = INVALID_SOCKET;
       fds[1] = INVALID_SOCKET;
-      if (Poll_SocketPair(FALSE, TRUE, fds) < 0)
+      if (Poll_SocketPair(FALSE, TRUE, fds, 0) < 0)
    #else
       close(fds[0]);
       close(fds[1]);
@@ -2217,7 +2337,7 @@ PollUnitTest_StateMachine(void *clientData) // IN: Unused
 
       /* Make fds[1] both readable and writable. */
    #ifdef _WIN32
-      send(fds[0], (const char *)fds, sizeof fds, 0);
+      retval = send(fds[0], (const char *)fds, sizeof fds, 0);
    #else
       retval = write(fds[0], fds, 1);
    #endif
@@ -2406,7 +2526,7 @@ PollUnitTest(Bool vmx)  // IN: use vmx-size poll queue
    }
    fds[0] = INVALID_SOCKET;
    fds[1] = INVALID_SOCKET;
-   if (Poll_SocketPair(FALSE, TRUE, fds) < 0) {
+   if (Poll_SocketPair(FALSE, TRUE, fds, 0) < 0) {
 #else
    fds[0] = -1;
    fds[1] = -1;
