@@ -1,5 +1,6 @@
 /*********************************************************
- * Copyright (c) 2006-2023 VMware, Inc. All rights reserved.
+ * Copyright (c) 2006-2024 Broadcom. All rights reserved.
+ * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -37,6 +38,7 @@
 #include <time.h>
 #include <unistd.h>
 #include "str.h"
+#include "vm_basic_defs.h"
 
 #include "mspackWrapper.h"
 #include "deployPkg/deployPkgFormat.h"
@@ -1758,7 +1760,7 @@ ExtractZipPackage(const char* pkgName,
    Bool ret = TRUE;
 
    // strip the header from the file
-   snprintf(zipName, sizeof zipName, "%s/%x", destDir, (unsigned int)time(0));
+   snprintf(zipName, sizeof zipName, "%s/%llx", destDir, (long long)time(NULL));
    zipName[(sizeof zipName) - 1] = '\0';
    if ((pkgFd = open(pkgName, O_RDONLY)) < 0) {
       sLog(log_error, "Failed to open package file '%s' for read. (%s)",
@@ -2102,6 +2104,7 @@ DeployPkg_DeployPackageFromFile(const char* file)
           */
          sLog(log_info,
               "Deployment delegated to Cloud-init. Returning success.");
+         VMW_FALLTHROUGH();
       case DEPLOYPKG_STATUS_SUCCESS:
          retStatus = 0;
          break;
