@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2017,2019 VMware, Inc. All rights reserved.
+ * Copyright (c) 2010-2017,2019-2022 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -304,7 +304,7 @@ RandR12GetInfo(Display *display,     // IN: Pointer to our display connection
    }
 
    /*
-    * Sanity checks. This should never really happen with current drivers.
+    * Confidence checks. This should never really happen with current drivers.
     */
 
    if (nVMWOutput != info->nOutput) {
@@ -341,7 +341,7 @@ RandR12GetInfo(Display *display,     // IN: Pointer to our display connection
    }
 
    /*
-    * Sanity check. This should never really happen with our drivers.
+    * Confidence check. This should never really happen with our drivers.
     */
 
    if (numVMWCrtc != nVMWOutput) {
@@ -361,17 +361,17 @@ RandR12GetInfo(Display *display,     // IN: Pointer to our display connection
  *
  * RandR12CrtcDisable --
  *
- *      Disable crtcs and associated outputs before an fb size change.
- *      The function disables crtcs and associated outputs
+ *      Deactivate crtcs and associated outputs before an fb size change.
+ *      The function deactivates crtcs and associated outputs
  *      1) whose scanout area is too big for the new fb size.
- *      2) that are going to be disabled with the new topology.
+ *      2) that are going to be deactivated with the new topology.
  *
  * Results:
  *      TRUE on success, FALSE on failure.
  *
  * Side effects:
  *      The RandR12info context is modified.
- *      The current mode of disabled outputs is set to "None".
+ *      The current mode of deactivated outputs is set to "None".
  *
  *-----------------------------------------------------------------------------
  */
@@ -779,6 +779,7 @@ RandR12SetupOutput(Display *display,        // IN: The display connection
                    int height)              // IN: Height of scanout area
 {
    RRCrtc crtcID = info->xrrRes->crtcs[rrOutput->crtc];
+   XRRCrtcInfo *crtcInfo = info->crtcs[rrOutput->crtc];
    XRRModeInfo *mode;
    Status ret;
 
@@ -791,9 +792,14 @@ RandR12SetupOutput(Display *display,        // IN: The display connection
    if (!mode) {
       return FALSE;
    }
+   if (!crtcInfo) {
+       g_warning("%s: Wasn't able to find crtc info for crtc id %d.\n", __func__,
+                   (int)crtcID);
+       return FALSE;
+   }
 
    ret = XRRSetCrtcConfig(display, info->xrrRes, crtcID, CurrentTime, x, y,
-                          mode->id, RR_Rotate_0, &rrOutput->id, 1);
+                          mode->id, crtcInfo->rotation, &rrOutput->id, 1);
    if (ret == Success) {
       rrOutput->mode = mode->id;
       return TRUE;
@@ -908,7 +914,6 @@ RandR12Revert(Display *display,    // IN: The display connection
    unsigned int i;
    RandR12Info *info = *pInfo;
    XRRScreenResources *xrrRes = info->xrrRes;
-   RandR12Output *rrOutput;
    XRRCrtcInfo *crtc;
    RRCrtc crtcID;
 
@@ -916,7 +921,7 @@ RandR12Revert(Display *display,    // IN: The display connection
 
    for (i = 0; i < info->nOutput; ++i) {
 
-      rrOutput = &info->outputs[i];
+      RandR12Output *rrOutput = &info->outputs[i];
       crtc = info->crtcs[rrOutput->crtc];
       crtcID = xrrRes->crtcs[rrOutput->crtc];
 
